@@ -4,9 +4,9 @@
 dbus_bool_t DBusAddWatch(DBusWatch *watch, void *data)
 {
     FcitxDBusWatch *w;
-    FcitxDBusWatch **watches = (FcitxDBusWatch**) data;
+    FcitxDBusWatchList *watches = (FcitxDBusWatchList*) data;
 
-    for (w = *watches; w; w = w->next) {
+    for (w = watches->head; w; w = w->next) {
         if (w->watch == watch) {
             return TRUE;
         }
@@ -16,8 +16,9 @@ dbus_bool_t DBusAddWatch(DBusWatch *watch, void *data)
         return FALSE;
 
     w->watch = watch;
-    w->next = *watches;
-    *watches = w;
+    w->next = watches->head;
+    watches->head = w;
+    watches->listModified = 1;
     return TRUE;
 }
 
@@ -25,29 +26,30 @@ void DBusRemoveWatch(DBusWatch *watch, void *data)
 {
     FcitxDBusWatch *w;
     FcitxDBusWatch *next, *prev;
-    FcitxDBusWatch **watches = (FcitxDBusWatch**)data;
+    FcitxDBusWatchList *watches = (FcitxDBusWatchList*)data;
 
     prev = NULL;
-    for (w = *watches;w;w = next) {
+    for (w = watches->head; w; w = next) {
         next = w->next;
         if (w->watch == watch) {
             free(w);
             if (prev) {
                 prev->next = next;
             } else {
-                *watches = next;
+                watches->head = next;
             }
+            watches->listModified = 1;
         } else {
             prev = w;
         }
     }
 }
 
-int DBusUpdateFDSet(FcitxDBusWatch* watches, fd_set* rfds, fd_set* wfds, fd_set* efds)
+int DBusUpdateFDSet(FcitxDBusWatchList* watches, fd_set* rfds, fd_set* wfds, fd_set* efds)
 {
     int maxfd = 0;
     FcitxDBusWatch* w;
-    for (w = watches; w; w = w->next)
+    for (w = watches->head; w; w = w->next)
         if (dbus_watch_get_enabled(w->watch)) {
             unsigned int flags = dbus_watch_get_flags(w->watch);
             int fd = dbus_watch_get_unix_fd(w->watch);
@@ -67,28 +69,33 @@ int DBusUpdateFDSet(FcitxDBusWatch* watches, fd_set* rfds, fd_set* wfds, fd_set*
     return maxfd;
 }
 
-void DBusProcessEventForWatches(FcitxDBusWatch* watches, fd_set* rfds, fd_set* wfds, fd_set* efds)
+void DBusProcessEventForWatches(FcitxDBusWatchList* watches, fd_set* rfds, fd_set* wfds, fd_set* efds)
 {
     FcitxDBusWatch *w;
 
-    for (w = watches; w; w = w->next) {
-        if (dbus_watch_get_enabled(w->watch)) {
-            unsigned int flags = 0;
-            int fd = dbus_watch_get_unix_fd(w->watch);
+    do {
+        watches->listModified = 0;
+        FcitxDBusWatch *next = NULL;
+        for (w = watches->head; !watches->listModified && w; w = next) {
+            next = w->next;
+            if (dbus_watch_get_enabled(w->watch)) {
+                unsigned int flags = 0;
+                int fd = dbus_watch_get_unix_fd(w->watch);
 
-            if (FD_ISSET(fd, rfds))
-                flags |= DBUS_WATCH_READABLE;
+                if (FD_ISSET(fd, rfds))
+                    flags |= DBUS_WATCH_READABLE;
 
-            if (FD_ISSET(fd, wfds))
-                flags |= DBUS_WATCH_WRITABLE;
+                if (FD_ISSET(fd, wfds))
+                    flags |= DBUS_WATCH_WRITABLE;
 
-            if (FD_ISSET(fd, efds))
-                flags |= DBUS_WATCH_ERROR;
+                if (FD_ISSET(fd, efds))
+                    flags |= DBUS_WATCH_ERROR;
 
-            if (flags != 0)
-                dbus_watch_handle(w->watch, flags);
+                if (flags != 0)
+                    dbus_watch_handle(w->watch, flags);
+            }
         }
-    }
+    } while (watches->listModified);
 }
 
 void DBusProcessEventForConnection(DBusConnection* connection)
