@@ -1,5 +1,6 @@
 #include <assert.h>
 #include "fcitx-utils/utils.h"
+#include "fcitx/fcitx.h"
 
 int main()
 {
@@ -7,6 +8,8 @@ int main()
     utarray_init(&array, &ut_int_icd);
     int test[] = {3, 1, 2};
     int test2[] = {1, 2, 3};
+    FCITX_UNUSED(test);
+    FCITX_UNUSED(test2);
     int i;
     i = 1;
     utarray_push_back(&array, &i);
@@ -17,17 +20,17 @@ int main()
     utarray_move(&array, 2, 0);
     {
         utarray_foreach(p, &array, int) {
-            assert(*p = test[utarray_eltidx(&array, p)]);
+            assert(*p == test[utarray_eltidx(&array, p)]);
         }
     }
     utarray_move(&array, 0, 2);
 
     {
         utarray_foreach(p, &array, int) {
-            assert(*p = test2[utarray_eltidx(&array, p)]);
+            assert(*p == test2[utarray_eltidx(&array, p)]);
         }
     }
     utarray_done(&array);
 
     return 0;
-}
\ No newline at end of file
+}
